'use strict';

function _interopDefault (ex) { return (ex && (typeof ex === 'object') && 'default' in ex) ? ex['default'] : ex; }

var fs = require('fs');
var path = _interopDefault(require('path'));
var rollupPluginutils = require('rollup-pluginutils');
var crypto = _interopDefault(require('crypto'));
var mime = _interopDefault(require('mime'));
var mkpath = _interopDefault(require('mkpath'));

function asyncGeneratorStep(gen, resolve, reject, _next, _throw, key, arg) {
  try {
    var info = gen[key](arg);
    var value = info.value;
  } catch (error) {
    reject(error);
    return;
  }

  if (info.done) {
    resolve(value);
  } else {
    Promise.resolve(value).then(_next, _throw);
  }
}

function _asyncToGenerator(fn) {
  return function () {
    var self = this,
        args = arguments;
    return new Promise(function (resolve, reject) {
      var gen = fn.apply(self, args);

      function _next(value) {
        asyncGeneratorStep(gen, resolve, reject, _next, _throw, "next", value);
      }

      function _throw(err) {
        asyncGeneratorStep(gen, resolve, reject, _next, _throw, "throw", err);
      }

      _next(undefined);
    });
  };
}

function _slicedToArray(arr, i) {
  return _arrayWithHoles(arr) || _iterableToArrayLimit(arr, i) || _nonIterableRest();
}

function _arrayWithHoles(arr) {
  if (Array.isArray(arr)) return arr;
}

function _iterableToArrayLimit(arr, i) {
  var _arr = [];
  var _n = true;
  var _d = false;
  var _e = undefined;

  try {
    for (var _i = arr[Symbol.iterator](), _s; !(_n = (_s = _i.next()).done); _n = true) {
      _arr.push(_s.value);

      if (i && _arr.length === i) break;
    }
  } catch (err) {
    _d = true;
    _e = err;
  } finally {
    try {
      if (!_n && _i["return"] != null) _i["return"]();
    } finally {
      if (_d) throw _e;
    }
  }

  return _arr;
}

function _nonIterableRest() {
  throw new TypeError("Invalid attempt to destructure non-iterable instance");
}

const promisily = fn => (...args) => {
  return new Promise((resolve, reject) => {
    fn(...args, (err, res) => {
      if (err) {
        return reject(err);
      }

      return resolve(res);
    });
  });
};

function copy(src, dest) {
  return new Promise((resolve, reject) => {
    const read = fs.createReadStream(src);
    read.on('error', reject);
    const write = fs.createWriteStream(dest);
    write.on('error', reject);
    write.on('finish', resolve);
    read.pipe(write);
  });
} // https://github.com/filamentgroup/directory-encoder/blob/master/lib/svg-uri-encoder.js


function encodeSVG(buffer) {
  return encodeURIComponent(buffer.toString('utf-8') // strip newlines and tabs
  .replace(/[\n\r]/gim, '').replace(/\t/gim, ' ') // strip comments
  .replace(/<!\-\-(.*(?=\-\->))\-\->/gim, '') // replace
  .replace(/'/gim, '\\i')) // encode brackets
  .replace(/\(/g, '%28').replace(/\)/g, '%29');
}

const _stat = promisily(fs.stat);

const _readFile = promisily(fs.readFile);

const _mkpath = promisily(mkpath);

function image(options = {}) {
  const _options$limit = options.limit,
        limit = _options$limit === void 0 ? 8 * 1024 : _options$limit,
        _options$fileName = options.fileName,
        fileName = _options$fileName === void 0 ? '[hash][extname]' : _options$fileName,
        _options$include = options.include,
        include = _options$include === void 0 ? ['**/*.svg', '**/*.png', '**/*.jpg', '**/*.gif'] : _options$include,
        exclude = options.exclude,
        _options$output = options.output,
        output = _options$output === void 0 ? '' : _options$output,
        _options$hash = options.hash,
        _options$reserveImpor = options.reserveImportInJs,
        reserveImportInJs = _options$reserveImpor === void 0 ? true : _options$reserveImpor,
        _options$compatibleWi = options.compatibleWithCss;
  const filter = rollupPluginutils.createFilter(include, exclude);
  const copies = Object.create(null);
  return {
    name: 'image',

    load(id) {
      var _this = this;

      return _asyncToGenerator(function* () {
        if (!filter(id)) return null;

        const _ref = yield Promise.all([_stat(id), _readFile(id)]),
              _ref2 = _slicedToArray(_ref, 2),
              stats = _ref2[0],
              buffer = _ref2[1];

        let data;

        if (limit && stats.size > limit || limit === 0) {
          const hash = crypto.createHash('sha1').update(buffer).digest('hex').substr(0, 16);
          const ext = path.extname(id);
          const name = path.basename(id, ext);
          let outputFileName = fileName.replace(/\[hash\]/g, hash).replace(/\[extname\]/g, ext).replace(/\[name\]/g, name);

          if (!!output) {
            // options 中指定的 output 为 rollup 中 output.file 的相对路径
            // 此处直接拼接，在复制图片的过程中会拼接 output.file 的路径，所以并无问题
            const separator = '/';
            const isEndWithSep = output.endsWith(separator);
            outputFileName = `${output}${isEndWithSep ? '' : separator}${outputFileName}`;
          } // generateBundle 复制 copies 中的文件


          copies[id] = outputFileName; // reserveImportInJs 为 true 并且资源文件大小超过 limit 时，
          // 会依旧保留其import(或者 require) 语句

          if (reserveImportInJs) {
            const code = `export default require('${outputFileName}');`;

            const ast = _this.parse(`var a = require('a')`); // 只是需要这样的 ast 而已


            return {
              ast,
              code,
              map: {
                mappings: ''
              }
            };
          }

          data = `${outputFileName}`;
        } else {
          // transform to base64
          const mimetype = mime.getType(id);
          const isSVG = mimetype === 'image/svg+xml';
          data = isSVG ? encodeSVG(buffer) : buffer.toString('base64');
          const encoding = isSVG ? '' : ';base64';
          data = `data:${mimetype}${encoding},${data}`;
        }

        return `export default "${data}"`;
      })();
    },

    generateBundle(outputOptions, bundle, isWrite) {
      return _asyncToGenerator(function* () {
        const file = outputOptions.file; // like dist/es

        const base = path.dirname(file);
        yield _mkpath(base);
        return Promise.all(Object.keys(copies).map(
        /*#__PURE__*/
        function () {
          var _ref3 = _asyncToGenerator(function* (name) {
            const output = copies[name]; // Create a nested directory if the fileName pattern contains
            // a directory structure

            const outputDirectory = path.join(base, path.dirname(output));
            yield _mkpath(outputDirectory);
            return copy(name, path.join(base, output));
          });

          return function (_x) {
            return _ref3.apply(this, arguments);
          };
        }()));
      })();
    }

  };
}

module.exports = image;
